#include <iostream>
#include <vector>
#include <fstream>
#include <sstream>

using namespace std;

//! Converts string to most types, specially numbers.
/*!
  \param input string to be converted.
  \return 'input' converted to 'T'.
*/
template <class T>
void to_num(std::string s, T& num)
{
  std::istringstream str(s);
  str >> num;
}


int main(int argc, char* argv[])
{

  if ( (argc!=4) && (argc!=5) && (argc!=6) )
    {
      cout << string("Program \"") + argv[0]
        + "\" requires three, four or five arguments:\n"
        + "[input file] [limit distance] [output file]\n"
	+ "or\n"
	+ "[points] [curve lengths] [limit distance] [output file]\n"
	+ "or\n"
	+ "[points] [curve lengths] [limit distance] [fronts file] [output file]" << endl;
      return 0;
    }

  string points_file(argv[1]);
  double distance_max;
  string lengths_file;
  string nb_fronts_file;
  string output_file(argv[argc-1]);
  if (argc==4)
    to_num(argv[2], distance_max);
  else if (argc==5)
    {
      lengths_file = argv[2];
      to_num(argv[3], distance_max);
    }
  else
    {
      lengths_file = argv[2];
      to_num(argv[3], distance_max);
      nb_fronts_file = argv[4];
    }

  double A, B, C, D, dist, A0, B0;
  ifstream points_stream(points_file.c_str());
  ofstream output_stream(output_file.c_str());
  ofstream nb_fronts_stream;

  if (argc==4)
    {
      points_stream >> A;
      points_stream >> B;
      A0 = A;
      B0 = B;
      if (points_stream.good())
	output_stream << A << "   " << B << endl;
      while (points_stream.good())
	{
	  points_stream >> C;
	  if (points_stream.good())
	    {
	      points_stream >> D;
	      if (sqrt( (A-C)*(A-C) + (B-D)*(B-D) )
		  < distance_max)
		output_stream << C << "   " << D << endl;
	      else
		{
		  if (sqrt( (A0-A)*(A0-A) + (B0-B)*(B0-B) )
		      < distance_max)
		    output_stream << A0 << "   " << B0 << endl;
		  output_stream << endl << endl;
		  output_stream << C << "   " << D << endl;
		  A0 = C;
		  B0 = D;
		}
	      A = C;
	      B = D;
	    }
	}
    }
  else  // (argc==5) || (argc==6)
    {
      ifstream lengths_stream(lengths_file.c_str());
      int length;
      vector<int> lengths(0);
      while (lengths_stream.good())
	{
	  string str_nb;	  
	  getline(lengths_stream, str_nb);
	  bool num(false); int i(0);
	  while ( (!num) && (i<str_nb.size()) )
	    {
	      num = num || isdigit(str_nb[i]);
	      i++;
	    }
	  if (num)
	    {
	      to_num(str_nb, length);
	      lengths.insert(lengths.end(), length);
	    }
	}
      
      int pos(1), index(0);
      int nb_fronts(1);

      if (argc==6)
	nb_fronts_stream.open(nb_fronts_file.c_str());
      
      points_stream >> A;
      points_stream >> B;
      A0 = A;
      B0 = B;
      if (points_stream.good())
	output_stream << A << "   " << B << endl;
      while ( points_stream.good() && (index<lengths.size()) )
	{
	  points_stream >> C;
	  if (points_stream.good())
	    {
	      points_stream >> D;
	      if (pos==lengths[index])
		{
		  if (sqrt( (A0-A)*(A0-A) + (B0-B)*(B0-B) )
		      < distance_max)
		    output_stream << A0 << "   " << B0 << endl;
		  output_stream << endl << endl;
		  output_stream << C << "   " << D << endl;
		  A0 = C;
		  B0 = D;
		  pos = 0;
		  index++;
		  if (argc==6)
		    {
		      nb_fronts_stream << nb_fronts << endl;
		      nb_fronts = 1;
		    }
		}
	      else if (sqrt( (A-C)*(A-C) + (B-D)*(B-D) )
		       < distance_max)
		output_stream << C << "   " << D << endl;
	      else
		{
		  if (sqrt( (A0-A)*(A0-A) + (B0-B)*(B0-B) )
		      < distance_max)
		    output_stream << A0 << "   " << B0 << endl;
		  output_stream << endl << endl;
		  output_stream << C << "   " << D << endl;
		  A0 = C;
		  B0 = D;
		  nb_fronts++;
		}
	      A = C;
	      B = D;
	      pos++;
	    }
	}
    }
  
  if (argc==6)
    nb_fronts_stream.close();
  points_stream.close();

  return 0;

}
