/*
ImageAnalyzer.java
Written by Blake Shaw 5/04
*/

import java.util.*;
import java.awt.*;
import javax.imageio.ImageIO;
import java.io.*;
import java.awt.image.BufferedImage;
import java.lang.*;


public class ImageAnalyzer {

	//Explains Usage for using as a command line tool with flags
	public static void badUsage(){
		System.out.println("Incorrect input");
		System.out.println("Usage: java ImageAnalyzer OutputOption GradientOption FilterOption InputFile OutputFile");
		System.out.println("Output Options:");
		System.out.println("\t-d --> creates data file");
		System.out.println("\t-i --> writes out image files at various stages");
		System.out.println("\t-di --> creates both image and data files");
		System.out.println("Gradient Options:");
		System.out.println("\t-s --> use sobel operators");
		System.out.println("\t-l --> use laplace operators");
		System.out.println("\t-lt --> use laplace operators with threshold overlay");
		System.out.println("\t-n --> do not use gradient");
		System.out.println("Filter Options:");
		System.out.println("\t-n --> no filters");
		System.out.println("\t-g --> gaussian");
		System.out.println("\t-rmb --> use red minus blue color channel filter");
		System.out.println("\t-allf --> gaussian and color channel filter");
		System.exit(1);
	}
	
	//Main method for when used as a command line tool with flags
	public static void main(String args[]) {
		if(args.length != 5){
			badUsage();
		}
		
		String filename = args[3];
		File theFile = new File(filename);
		try {
			BufferedImage bi = ImageIO.read(theFile);
			
			
			
			if (bi == null) {
				System.out.println("Reading from file failed.");
				System.exit(1);
			}
			
			
			BufferedImage grad = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
			boolean writeAll = false;
			if(args[0].equals("-i") || args[0].equals("-di")){
				writeAll = true;
			}
			
			if(args[2].equals("-n")){
				
			} else if(args[2].equals("-g")){
				bi = computeGaussian(bi);
				if (writeAll) ImageIO.write(bi, "png", new File(args[4] + "Gaussian.png"));
			} else if(args[2].equals("-rmb")){
				bi = redMinusBlueFilter(bi);
				if (writeAll) ImageIO.write(bi, "png", new File(args[4] + "Filtered.png"));
			} else if(args[2].equals("-allf")){
				bi = redMinusBlueFilter(bi);
				if (writeAll) ImageIO.write(bi, "png", new File(args[4] + "Filtered.png"));
				bi = computeGaussian(bi);
				if (writeAll) ImageIO.write(bi, "png", new File(args[4] + "Gaussian.png"));
			} else {
				badUsage();
			}
			
			if(args[1].equals("-s")){
				grad = computeGradient(bi);
				grad = computeGaussian(grad);
			} else if(args[1].equals("-l")){
				grad = computeGradient2(bi);
				grad = computeGaussian(grad);
			} else if(args[1].equals("-lt")){
				//grad = computeGradient2(bi);
				//grad = computeGaussian(grad);
				grad = threshold(bi, 100);
			} else if(args[1].equals("-n")){
				grad = bi;
				
			} else if(args[1].equals("-b")){
				grad = computeGradient(bi);
				grad = computeGradient(grad);
				grad = computeGaussian(grad);
			} else {
				badUsage();
			}
			
			if(args[0].equals("-d")){
				writeToBWDataFile(grad, args[4]);
			} else if (args[0].equals("-i")){
				ImageIO.write(grad, "png", new File(args[4] + ".png"));
			} else if (args[0].equals("-di")){
				writeToBWDataFile(grad, args[4]);
				ImageIO.write(grad, "png", new File(args[4] + ".png"));
			} else {
				badUsage();
			}
		} catch (IOException e){
			System.out.println(e);
		}
	}
	
	
	//Thresholds image bi at grayscale value t
	public static BufferedImage threshold(BufferedImage bi, int t){
		BufferedImage result = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		for (int y=0; y<bi.getHeight(); y++){
			for (int x=0; x<bi.getWidth(); x++){
				if(intensity(bi, x, y) > t){
					result.setRGB(x, y, (new Color(255, 255, 255)).getRGB());
				} else {
					result.setRGB(x, y, (new Color(0, 0, 0)).getRGB());
				}
			}
		}
		return result;
	}
	
	//Computes the Gradient of the image using Sobel Operators
	public static BufferedImage computeGradient(BufferedImage bi){
		BufferedImage result = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		int sum = 0;
		int[][] SobelX = {{-1, 0, 1}, {-2, 0, 2}, {-1, 0, 1}};
		int[][] SobelY = {{1, 2, 1}, {0, 0, 0}, {-1, -2, -1}};
				
		for (int y=0; y<bi.getHeight(); y++){
			for (int x=0; x<bi.getWidth(); x++){
				sum = 0;				
				
				//if not on boundries
				if( !(x == 0 || x == bi.getWidth()-1 || y == 0 || y == bi.getHeight()-1)){
					for(int i=-1; i<=1; i++){
						for(int j=-1; j<=1; j++){
							sum = sum + (SobelX[i+1][j+1] * intensity(bi, x+i, y+j));
							sum = sum + (SobelY[i+1][j+1] * intensity(bi, x+i, y+j));
						}
					}

				}
				if(sum > 255) sum = 255;
				if(sum < 0) sum = 0;
				result.setRGB(x, y, (new Color(255 - sum, 255 - sum, 255 - sum)).getRGB());
				//result.setRGB(x, y, (new Color(255, 255, 255)).getRGB());
			}
		}
		return result;
	}
	
	//Computes the Gradient of the image using Laplaician Operator
	public static BufferedImage computeGradient2(BufferedImage bi){
		BufferedImage result = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		int sum = 0;
		int[][] Laplace = {{-1, -1, -1, -1, -1}, {-1, -1, -1, -1, -1}, {-1, -1, 24, -1, -1}, {-1, -1, -1, -1, -1}, {-1, -1, -1, -1, -1}};

		for (int y=0; y<bi.getHeight(); y++){
			for (int x=0; x<bi.getWidth(); x++){
				sum = 0;				
				//if not on boundries
				if( !(x == 0 || x == 1 || x == bi.getWidth()-1 || x == bi.getWidth()-2 || y == 0 || y == 1 || y == bi.getHeight()-1 || y == bi.getHeight()-2)){
					for(int i=-2; i<=2; i++){
						for(int j=-2; j<=2; j++){
							sum = sum + (Laplace[i+2][j+2] * intensity(bi, x+i, y+j));
						}
					}

				}
				if(sum > 255) sum = 255;
				if(sum < 0) sum = 0;
				result.setRGB(x, y, (new Color(255 - sum, 255 - sum, 255 - sum)).getRGB());
			}
		}
		return result;
	}
	
	public static BufferedImage computeGaussianMultiple(BufferedImage bi, int max){
		BufferedImage result = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		result = bi;
		for(int i=0; i<max; i++){
			result = computeGaussian(result);
		}
		return result;
	}
	
	
	public static BufferedImage computeGaussian(BufferedImage bi){
		BufferedImage result = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		double sum = 0;
		int sumInt = 0;
		int[][] kernel = {{1, 4, 7, 4, 1}, {4, 16, 26, 16, 4}, {7, 26, 41, 26, 7}, {4, 16, 26, 16, 4}, {1, 4, 7, 4, 1}};
		
		for (int y=0; y<bi.getHeight(); y++){
			for (int x=0; x<bi.getWidth(); x++){
				sum = 0;				
				//if not on boundries
				if( !(x == 0 || x == 1 || x == bi.getWidth()-1 || x == bi.getWidth()-2 || y == 0 || y == 1 || y == bi.getHeight()-1 || y == bi.getHeight()-2)){
					for(int i=-2; i<=2; i++){
						for(int j=-2; j<=2; j++){
						
							sum = sum + (((double) kernel[i+2][j+2] / 273) * intensity(bi, x+i, y+j));
						}
					}
				}
				if(sum > 255) sum = 255;
				if(sum < 0) sum = 0;
				
				sumInt = (int) sum;
				result.setRGB(x, y, (new Color(sumInt, sumInt, sumInt)).getRGB());
			}
		}
		return result;
	}

	//Calculates an intensity value between 0-255 for each pixel
	//Accepts color images and grayscale
	public static int intensity(BufferedImage bi, int x, int y){
		Color c = new Color(bi.getRGB(x,y));
		return (int)Math.floor((c.getRed() + c.getGreen() + c.getBlue()) / 3);
	}
	
	//Writes out image to binary data file to be read by Multivac
	public static void writeToBWDataFile(BufferedImage bi, String filename){
		try {
			FileOutputStream fs = new FileOutputStream(filename);
			int cnum;
			for (int y=0; y<bi.getHeight(); y++){
				for (int x=0; x<bi.getWidth(); x++){
					cnum = (new Color(bi.getRGB(x,y))).getRed();
					fs.write(cnum);
				}
			}
			fs.close();
		} catch (IOException e){}
	}
	
	//Returns and image which is the red channel - the blue channel for bi 
	public static BufferedImage redMinusBlueFilter(BufferedImage bi){
		BufferedImage redminusblue = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		Color c;
		int cnum;
		for (int y=0; y<bi.getHeight(); y++){
			for (int x=0; x<bi.getWidth(); x++){
				c = new Color(bi.getRGB(x,y));
				cnum = c.getRed() - c.getBlue();
				if (cnum < 0) cnum = 0;
				redminusblue.setRGB(x, y, (new Color(cnum, cnum, cnum)).getRGB());
			}
		}
		return redminusblue;	
	}
	
	//Inverts the Grayscale values for an image
	public static BufferedImage invert(BufferedImage bi){
		BufferedImage inverted = new BufferedImage(bi.getWidth(), bi.getHeight(), BufferedImage.TYPE_INT_RGB);
		int cnum;
		for (int y=0; y<bi.getHeight(); y++){
			for (int x=0; x<bi.getWidth(); x++){
				cnum = 255 - intensity(bi, x, y);
				inverted.setRGB(x, y, (new Color(cnum, cnum, cnum)).getRGB());
			}
		}
		return inverted;	
	}
}