/*
MultivacFrontEnd.java

to compile:
	javac MultivacFrontEnd.java
to run:
	java MultivacFrontEnd
	
Written by Blake Shaw 6/04

*/


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.imageio.ImageIO;
import java.io.*;
import java.lang.*;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;

public class MultivacFrontEnd extends Frame implements ActionListener{

	//Gloabal Variables
	public Frame frm, frm2;
	public Panel panel;
	public Button b1, b2, b3, b4, b5, b6, b7, b8, b9, b10, b11, b12, b13;
	public Label title;
	public Vector paramGroups; //list of all parameters groups
	public Vector outlinePoints; //Points on the current curve
	public ParameterGroup general, segmentation, simulation, image, curves;
	public static Process process; //Used to run shell commands
	public String workingDir; //The current main directory
	public Canvas canvas; //The picture area
	public boolean userCurves; //True if using a user defined curve
	BufferedImage bi; //The image that is currently displayed in the canvas
	
		
	//Main method
   	public static void main (String arg[]){
		MultivacFrontEnd mfe = new MultivacFrontEnd();	
	}
	
	//Constructor
	public MultivacFrontEnd(){
		//Make new frame
		frm= new Frame("Multivac Controls");
		frm.setSize(800, 500);
		frm.setBackground (Color.lightGray);
		frm.setVisible(true);
		
		//Make new panel
		panel = new Panel();
		frm.add(panel);
		title = new Label("Multivac Parameter File Controller");
		title.setFont(new Font("Helvetica", Font.BOLD, 20));
		panel.add(title);
		
		outlinePoints = new Vector();
		paramGroups = new Vector(); //Initialize Parameter Group List

		//General Parameters
		general = new ParameterGroup("General Parameters");
		general.addParameter(new Parameter("Working Directory", figureOutDirectory(), "BaseDirectory")); //Note: The working directory must contain both a results directory and a directory for multivac
		general.addParameter(new Parameter("Parameter File Name", "test.mconf", "configFilename"));
		paramGroups.add(general);
		
		//Image Segmentation Parameters
		segmentation = new ParameterGroup("Image Segmentation Parameters");
		segmentation.addParameter(new Parameter("epsilon", "0.01", "epsilon_c"));
		segmentation.addParameter(new Parameter("boost", "30", "boost"));
		segmentation.addParameter(new Parameter("slowDown", "400", "slowDown"));
		segmentation.addParameter(new Parameter("Gaussian Iterations", "3", "gaussIters"));
		paramGroups.add(segmentation);
		
		//Multivac Simulation Parameters
		simulation = new ParameterGroup("Simulation Parameters");
		simulation.addParameter(new Parameter("Final Time", "0.03", "FinalTime"));
		simulation.addParameter(new Parameter("Time Step", "0.0001", "TimeStep"));
		simulation.addParameter(new Parameter("Grid Size", "512", "GridSize"));
		simulation.addParameter(new Parameter("Tube Semi Width", "12", "tubeSemiWidth"));
		simulation.addParameter(new Parameter("Number of Curves", "10", "numCurves"));
		paramGroups.add(simulation);
		
		//Adds each parameter to the gui
		for(int i=0; i<paramGroups.size(); i++){
			panel.add((ParameterGroup) paramGroups.elementAt(i));
		}
		
		//Set up buttons	
		b1 = new Button("Read Configuration File");
		b2 = new Button("Write Configuration File");
		b3 = new Button("Write and Run");
		b4 = new Button("Default Parameters");
		b13 = new Button("Backup Last Run");
		b1.addActionListener(this);
		b2.addActionListener(this);
		b3.addActionListener(this);
		b4.addActionListener(this);
		b13.addActionListener(this);
		panel.add(b1);
		panel.add(b2);
		panel.add(b3);
		panel.add(b4);
		panel.add(b13);
		
		
		
		//Set up Image Controller Frame
		frm2= new Frame("Multivac Image Segmentation");
		frm2.setSize(1010, 550);
		frm2.setLocation(300, 0);
		frm2.setBackground (Color.lightGray);
		frm2.setVisible(true);
		
		//Make new panel
		Panel panel2 = new Panel();
		Panel panel3 = new Panel();
		panel2.setLayout(new FlowLayout(FlowLayout.CENTER));
		panel3.setLayout(new FlowLayout(FlowLayout.CENTER));
		frm2.setLayout(new BorderLayout());
		frm2.add(panel3, BorderLayout.WEST);
		frm2.add(panel2, BorderLayout.CENTER);
		
		//Load Default Image and Setup Canvas
		loadImage("test.jpg");
		canvas = new Canvas() {public void paint(Graphics g) { draw(g); }};
		canvas.setSize(512, 512);
		canvas.addMouseListener(new MouseAdapter() {
      		public void mousePressed(MouseEvent me) {
      			Point point = new Point(me.getPoint());
      			outlinePoints.add(point);
				//System.out.println(point);
				userCurves = true;
				canvas.repaint();
      		}
      		public void mouseExited(MouseEvent me) {

      		}

      		public void mouseReleased(MouseEvent me) {

      		}

    	});
    	
    	//Setup Image Control Parameters and Buttons
    	image = new ParameterGroup("Image Controls");
    	curves = new ParameterGroup("Curve Controls");
    	paramGroups.add(image);
    	paramGroups.add(curves);
    	
		b5 = new Button("Generate Image Data");
		b6 = new Button("Generate Final Image");
		b7 = new Button("Load Image");
		b8 = new Button("Read Curves from Last Multivac Output");
		b12 = new Button("Read Curves from Intial Curve File");
		b9 = new Button("Write Curves on Screen as File");
		b10 = new Button("Delete Curve");
		b11 = new Button("Delete Last Point");
		
		b5.addActionListener(this);
		b6.addActionListener(this);
		b7.addActionListener(this);
		b8.addActionListener(this);
		b9.addActionListener(this);
		b10.addActionListener(this);
		b11.addActionListener(this);
		b12.addActionListener(this);
		
		panel3.add(canvas);
		
		image.addParameter(new Parameter("Image Name", "test.jpg", "ImageName", 10));
		image.addButton(b7);
		image.addSpace();
		image.addParameter(new Parameter("Image Data Filename", "test.dat", "ImageDataFileName", 20));
		image.addButton(b5);
		image.addSpace();
		image.addButton(b6);
		
		curves.addParameter(new Parameter("Initial Curve File", "init.curve", "intialCurveFile", 20));
		curves.addButton(b9);
		curves.addSpace();
		curves.addButton(b12);
		curves.addButton(b8);
		curves.addSpace();
		curves.addButton(b10);
		curves.addButton(b11);
				
		panel2.add(image);
		panel2.add(curves);

	}
	
	//	draw(Graphics g)
	//	called when canvas refreshes
	//	draws outline and picture
	public void draw(Graphics g){
		//System.out.println("Draw Called");
		g.drawImage(bi, 0, 0, Color.black, new myImageObserver());
		Point p, p1, pnext;
		p1 = new Point();
		pnext = new Point();
		for(int i=0; i<outlinePoints.size(); i++){
			p = ((Point) outlinePoints.elementAt(i));
			if(i==0) p1 = p;
			g.fillOval(p.x - 1, p.y - 1, 2, 2);
			
			if(outlinePoints.size() > 0){
				if(i == outlinePoints.size()-1){
					pnext = p1;
				} else {
					pnext = ((Point) outlinePoints.elementAt(i+1));
				}
			}
			if( ((p.x - pnext.x)*(p.x - pnext.x) + (p.y - pnext.y)*(p.y - pnext.y) < 2) || (userCurves)){
				g.drawLine(p.x, p.y, pnext.x, pnext.y);
			}
		}
	}
	
	//	loadImage(String filename)
	//	loads the image with filename
	//	relative to working directory
	public void loadImage(String filename){
		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
		try{
			File f = new File(workingDir + "/" + filename);
			bi = ImageIO.read(f);
		} catch (Exception e){
			System.out.println(e);
		}
	
	}
	
	//Handles actions for button clicks
	public void actionPerformed(ActionEvent event){
		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
		String filename = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(1)).getValue();
		Object source = event.getSource();
		if (source == b2){
			System.out.println("Write Configuration File");
			writeToFile();
		} else if (source == b3){
			System.out.println("Write and Run");
			writeAndRun();
		} else if (source == b1){
			System.out.println("Read Configuration File");
			readFromFile();
		} else if (source == b4){
			System.out.println("Default Parameters");
			setToDefault();
		} else if (source == b5){
			System.out.println("Generate Image Data");
			generateImageData();
			System.out.println("Generate Image Data Complete");
		} else if (source == b6){
			System.out.println("Generate Final Image");
			String imageName = ((Parameter)image.params.elementAt(0)).getValue();
			run("../Display/display.sh " + workingDir + " " + imageName + " " + "overlay.png");
			loadImage("overlay.png");
			readCurvesFromMultivacOutput();
			userCurves = false;
			canvas.repaint();
		} else if (source == b7){
			String f = ((Parameter)image.params.elementAt(0)).getValue();
			System.out.println("Load Image: " + f);	
			loadImage(f);
			canvas.repaint();
		} else if (source == b8){
			userCurves = false;
			readCurvesFromMultivacOutput();
		} else if (source == b9){
			writeCurves();
		} else if (source == b10){
			outlinePoints = new Vector();
			canvas.repaint();
		} else if (source == b11){
			System.out.println("Removing point: " + outlinePoints.elementAt(outlinePoints.size() - 1));
			outlinePoints.remove(outlinePoints.size() - 1);
			canvas.repaint();
		} else if (source == b12){
			userCurves = true;
			readCurvesFromInit();
		} else if (source == b13){
			System.out.println("Backing Up Last Run");	
			String imageName = ((Parameter)image.params.elementAt(0)).getValue();
			userCurves = false;
			readCurvesFromMultivacOutput();
			writeCurvesToFile("final.curves");
			run("../BackUpLastRun.sh overlay.png " + filename + " " + ((Parameter)curves.params.elementAt(0)).getValue() + " " + imageName.substring(0, imageName.lastIndexOf('.')) + "*.png");
		}

   }
   
   	//Processes Image with filename specified as the parameter "Image Name"
    public void generateImageData(){
    		String imageName = ((Parameter)image.params.elementAt(0)).getValue();
    		String imageDataName = ((Parameter)image.params.elementAt(1)).getValue();
    		int gaussITS = (new Integer(((Parameter)segmentation.params.elementAt(3)).getValue())).intValue();
    		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
    		String f = workingDir + "/" + imageName.substring(0, imageName.lastIndexOf('.'));
    		loadImage(imageName);
    	try{
	   		BufferedImage b2 = ImageAnalyzer.redMinusBlueFilter(bi);
	   		ImageIO.write(b2, "png", new File(f + "Filtered.png"));
	   		b2 = ImageAnalyzer.computeGaussian(b2);
	   		ImageIO.write(b2, "png", new File(f + "Gaussian.png"));
	   		b2  = ImageAnalyzer.computeGradient2(b2);
	   		b2 = ImageAnalyzer.computeGaussianMultiple(b2, gaussITS);
	   		ImageAnalyzer.writeToBWDataFile(b2,  workingDir + "/" + imageDataName);
	   		ImageIO.write(b2, "png", new File(f + "DataImage.png"));
    	} catch (Exception e){
    		System.out.println("Error Generating Image Data");
    	}
    }
   
	//Iterates through all parameters and writes them to file
	public void writeToFile(){
   		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
   		String filename = workingDir + "/" + ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(1)).getValue();
   		try {
   			FileWriter fw = new FileWriter(filename);
			fw.write("# Multivac Test Configuration File");
			fw.write("\n");
			fw.write("# --------------------------------");
	   		fw.write("\n");
	   		
			for(int i=0; i<paramGroups.size(); i++){
				((ParameterGroup) paramGroups.elementAt(i)).writeToFile(fw);
			}
			
			fw.write("#");
			fw.close();
		} catch (IOException e){
			System.out.println("Error writing configuration file");
			System.out.println(e);
		}
	}
	
	//Write curves on the canvas to a file   
	public void writeCurvesToFile(String filename){
   		System.out.println("Writing Curves");
   		
   		if(outlinePoints.size() == 0){
   			System.out.println("No curves have been drawn or loaded to the screen");
   		} else {
   			workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
   			
   			try {
   				FileWriter fw = new FileWriter(workingDir + "/" + filename); 	
	   			float f;   		
		   		for(int i=0; i < outlinePoints.size(); i++){
		   			f = ((float)((Point)outlinePoints.elementAt(i)).x)/512 * 3;
		   			fw.write((new Float(f)).toString());
		   			fw.write(" ");
		   			f = ((float)((Point)outlinePoints.elementAt(i)).y)/512 * 3;
		   			fw.write((new Float(f)).toString());
		   			fw.write("\n");
		   		}
		   		
		   		f = ((float)((Point)outlinePoints.elementAt(0)).x)/512 * 3;
		   		fw.write((new Float(f)).toString());
		   		fw.write(" ");
		   		f = ((float)((Point)outlinePoints.elementAt(0)).y)/512 * 3;
		   		fw.write((new Float(f)).toString());
		   		
		   		fw.close();
	   		} catch (IOException e){
		   		System.out.println("Error writing curves file");
				System.out.println(e);
	   		}
   		}
   	}
   	
   	//Write curves to file specified by the parameter "Initial Curve File"
   	public void writeCurves(){
   		writeCurvesToFile(((Parameter)curves.params.elementAt(0)).getValue());
   	}
   	
   	//Read curves to screen from "Initial Curve File"
   	public void readCurvesFromInit(){
   		System.out.println("Reading Curves from Initial Curve File");
   		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
   		String filename = ((Parameter)curves.params.elementAt(0)).getValue();
   		
   		try{
			FileReader fr = new FileReader(workingDir + "/" + filename);
			BufferedReader br = new BufferedReader(fr);
			String s;
			StringTokenizer st;
			outlinePoints = new Vector();
			int x, y;
			double d;
			
			while((s = br.readLine()) != null){
				st = new StringTokenizer(s);
				x = (new Double(Double.parseDouble(st.nextToken()) * 512 / 3)).intValue();
				y = (new Double(Double.parseDouble(st.nextToken()) * 512 / 3)).intValue();
				outlinePoints.add(new Point(x, y));
			}
			canvas.repaint();
		} catch (Exception e) {
			System.out.println("Error reading curves file");
			System.out.println(e);
		}
   	}
   	
   	//Read curves from last Multivac output
   	public void readCurvesFromMultivacOutput(){
   		System.out.println("Reading Curves from Multivac Output File");
   		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
		try{
			FileReader fr = new FileReader(workingDir + "/Results/CurveLengths");
			BufferedReader br = new BufferedReader(fr);
			String s;
			StringTokenizer st;
			int numBeforeLastCurve = 0;
			int lastCurveSize = 0;
			
			while((s = br.readLine()) != null){
				st = new StringTokenizer(s);
				numBeforeLastCurve += lastCurveSize;
				lastCurveSize = Integer.parseInt(st.nextToken());
			}
			
			//System.out.println("numBeforeLast: " + numBeforeLastCurve + ", lastCurveSize: " + lastCurveSize);
			
			fr = new FileReader(workingDir + "/Results/Curves");
			br = new BufferedReader(fr);
			outlinePoints = new Vector();
			
			for(int i=0; i<numBeforeLastCurve; i++){
				s = br.readLine();
			}
			
			//System.out.println("Last line before last curve: " + s);
			
			int x, y;
			for(int i=0; i<lastCurveSize; i++){
				st = new StringTokenizer(br.readLine());
				x = (new Double(Double.parseDouble(st.nextToken()) * 512 / 3)).intValue();
				y = (new Double(Double.parseDouble(st.nextToken()) * 512 / 3)).intValue();
				outlinePoints.add(new Point(x, y));
			}
				outlinePoints.add((Point) outlinePoints.elementAt(0));
			canvas.repaint();
		} catch (Exception e) {
			System.out.println("Error reading curves file");
			System.out.println(e);
		}
   	}
   	
   	
   	//Reads a configuration file back into the gui
	public void readFromFile(){
		workingDir = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(0)).getValue();
   		String filename = workingDir + "/" + ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(1)).getValue();
   		
		try{
			FileReader fr = new FileReader(filename);
			BufferedReader br = new BufferedReader(fr);
			for(int i=0; i<paramGroups.size(); i++){
				((ParameterGroup) paramGroups.elementAt(i)).readFromFile(br);
			}
			fr.close();
		} catch (IOException e) {
			System.out.println("Error reading configuration file");
			System.out.println(e);
		}

   }
   
   
   //Sets parameters to their default value
   public void setToDefault(){
   		for(int i=0; i<paramGroups.size(); i++){
			((ParameterGroup) paramGroups.elementAt(i)).setToDefault();
		}
   }
   
   	//Writes configuration file and then runs multivac using it
	public void writeAndRun(){
		writeToFile();
		String filename = ((Parameter)((ParameterGroup) paramGroups.elementAt(0)).params.elementAt(1)).getValue();
		//run("ls");
		run("make preRunClean");
		String command = "../Multivac-2004-04-19/track " + workingDir + "/" + filename;
		System.out.println("Running Multivac: " + command);
		run(command);
		
	}

	public String figureOutDirectory(){
		String s = runS("pwd");
		s = s.substring(0, s.lastIndexOf("/"));
		return s;
	}

	//Executes a given string as a shell command -- output to terminal
	public static void run(String s){
		try{
    		process  = Runtime.getRuntime().exec(s);
        	BufferedReader is = new BufferedReader(new InputStreamReader(process.getInputStream()));
			String line;

    		while ((line = is.readLine()) != null){
      			System.out.println(line);
  			}

   	 	} catch (IOException i){}
	}
	
	//Executes a given string as a shell command -- output as string
	public static String runS(String s){
		String output = "";
		try{
    		process  = Runtime.getRuntime().exec(s);
        	BufferedReader is = new BufferedReader(new InputStreamReader(process.getInputStream()));
			String line;

    		while ((line = is.readLine()) != null){
      			output += line;
      			output += "\n";
  			}

   	 	} catch (IOException i){}
   	 	return output;
	}
	
		public class myImageObserver implements ImageObserver{
		public myImageObserver(){
	
		}
		public boolean imageUpdate(Image i, int infoflags, int x, int y, int width, int height){
			return false;
		}
	}
}
