// Copyright (C) 2001-2003 Vivien Mallet
//
// This file is part of Seldon library.
// Seldon library provides matrices and vectors structures for
// linear algebra.
// 
// Seldon is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Seldon is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.
//
// For more information, please see the Seldon home page:
//     http://spacetown.free.fr/lib/seldon/

#ifndef FILE_BLAS_2_CXX

/*
  Functions defined in this file:

  xROTG
*/

extern "C"
{
#include "cblas.h"
}

namespace Seldon
{


  /////////
  // Mlt //


  template <class T0,
	    class T1, class Prop1, class Storage1, class Allocator1,
	    class T2, class Storage2, class Allocator2,
	    class T3,
	    class T4, class Storage4, class Allocator4>
  void MltAdd(const T0 alpha,
	      const Matrix<T1, Prop1, Storage1, Allocator1>& M,
	      const Vector<T2, Storage2, Allocator2>& X,
	      const T3 beta,
	      Vector<T4, Storage4, Allocator4>& Y);


  /*** ColMajor and NoTrans ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const Matrix<float, Prop0, ColMajor, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sgemv(CblasColMajor, CblasNoTrans,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const Matrix<double, Prop0, ColMajor, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dgemv(CblasColMajor, CblasNoTrans,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<float> alpha,
	      const Matrix<complex<float>, Prop0, ColMajor, Allocator0>& A,
	      const Vector<complex<float>, Vect_Full, Allocator1>& X,
	      const complex<float> beta,
	      Vector<complex<float>, Vect_Full, Allocator2>& Y)
  {
    cblas_cgemv(CblasColMajor, CblasNoTrans,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<double> alpha,
	      const Matrix<complex<double>, Prop0, ColMajor, Allocator0>& A,
	      const Vector<complex<double>, Vect_Full, Allocator1>& X,
	      const complex<double> beta,
	      Vector<complex<double>, Vect_Full, Allocator2>& Y)
  {
    cblas_zgemv(CblasColMajor, CblasNoTrans,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  /*** ColMajor and TransA ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<float, Prop0, ColMajor, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sgemv(CblasColMajor, TransA,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<double, Prop0, ColMajor, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dgemv(CblasColMajor, TransA,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<float> alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<complex<float>, Prop0, ColMajor, Allocator0>& A,
	      const Vector<complex<float>, Vect_Full, Allocator1>& X,
	      const complex<float> beta,
	      Vector<complex<float>, Vect_Full, Allocator2>& Y)
  {
    cblas_cgemv(CblasColMajor, TransA,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<double> alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<complex<double>, Prop0, ColMajor, Allocator0>& A,
	      const Vector<complex<double>, Vect_Full, Allocator1>& X,
	      const complex<double> beta,
	      Vector<complex<double>, Vect_Full, Allocator2>& Y)
  {
    cblas_zgemv(CblasColMajor, TransA,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  /*** RowMajor and NoTrans ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const Matrix<float, Prop0, RowMajor, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sgemv(CblasRowMajor, CblasNoTrans,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const Matrix<double, Prop0, RowMajor, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dgemv(CblasRowMajor, CblasNoTrans,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<float> alpha,
	      const Matrix<complex<float>, Prop0, RowMajor, Allocator0>& A,
	      const Vector<complex<float>, Vect_Full, Allocator1>& X,
	      const complex<float> beta,
	      Vector<complex<float>, Vect_Full, Allocator2>& Y)
  {
    cblas_cgemv(CblasRowMajor, CblasNoTrans,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<double> alpha,
	      const Matrix<complex<double>, Prop0, RowMajor, Allocator0>& A,
	      const Vector<complex<double>, Vect_Full, Allocator1>& X,
	      const complex<double> beta,
	      Vector<complex<double>, Vect_Full, Allocator2>& Y)
  {
    cblas_zgemv(CblasRowMajor, CblasNoTrans,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  /*** RowMajor and TransA ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<float, Prop0, RowMajor, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sgemv(CblasRowMajor, TransA,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<double, Prop0, RowMajor, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dgemv(CblasRowMajor, TransA,
		A.GetM(), A.GetN(), alpha, A.GetData(), A.GetM(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<float> alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<complex<float>, Prop0, RowMajor, Allocator0>& A,
	      const Vector<complex<float>, Vect_Full, Allocator1>& X,
	      const complex<float> beta,
	      Vector<complex<float>, Vect_Full, Allocator2>& Y)
  {
    cblas_cgemv(CblasRowMajor, TransA,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const complex<double> alpha,
	      const enum CBLAS_TRANSPOSE TransA,
	      const Matrix<complex<double>, Prop0, RowMajor, Allocator0>& A,
	      const Vector<complex<double>, Vect_Full, Allocator1>& X,
	      const complex<double> beta,
	      Vector<complex<double>, Vect_Full, Allocator2>& Y)
  {
    cblas_zgemv(CblasRowMajor, TransA,
		A.GetM(), A.GetN(), reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(A.GetData()), A.GetM(),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(&beta),
		reinterpret_cast<void*>(Y.GetData()), 1);
  }


  //////////////////////////////////////////////////////////////////////////////////////

  /*** ColSymPacked and Upper ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const Matrix<float, Prop0, ColSymPacked, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sspmv(CblasColMajor, CblasUpper,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const Matrix<double, Prop0, ColSymPacked, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dspmv(CblasColMajor, CblasUpper,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  /*** ColSymPacked and Uplo ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const enum CBLAS_UPLO Uplo,
	      const Matrix<float, Prop0, ColSymPacked, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sspmv(CblasColMajor, Uplo,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const enum CBLAS_UPLO Uplo,
	      const Matrix<double, Prop0, ColSymPacked, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dspmv(CblasColMajor, Uplo,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  /*** RowSymPacked and Upper ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const Matrix<float, Prop0, RowSymPacked, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sspmv(CblasRowMajor, CblasUpper,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const Matrix<double, Prop0, RowSymPacked, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dspmv(CblasRowMajor, CblasUpper,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  /*** RowSymPacked and Uplo ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const float alpha,
	      const enum CBLAS_UPLO Uplo,
	      const Matrix<float, Prop0, RowSymPacked, Allocator0>& A,
	      const Vector<float, Vect_Full, Allocator1>& X,
	      const float beta,
	      Vector<float, Vect_Full, Allocator2>& Y)
  {
    cblas_sspmv(CblasRowMajor, Uplo,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void MltAdd(const double alpha,
	      const enum CBLAS_UPLO Uplo,
	      const Matrix<double, Prop0, RowSymPacked, Allocator0>& A,
	      const Vector<double, Vect_Full, Allocator1>& X,
	      const double beta,
	      Vector<double, Vect_Full, Allocator2>& Y)
  {
    cblas_dspmv(CblasRowMajor, Uplo,
		A.GetM(), alpha, A.GetData(),
		X.GetData(), 1, beta, Y.GetData(), 1);
  }


  // Mlt //
  /////////



  /////////////////
  // RANK1UPDATE //


  /*** ColMajor ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator1>& X,
		   const Vector<float, Vect_Full, Allocator2>& Y,
		   Matrix<float, Prop0, ColMajor, Allocator0>& A)
  {
    cblas_sger(CblasColMajor, A.GetM(), A.GetN(), alpha, X.GetData(), 1,
	       Y.GetData(), 1, A.GetData(), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator1>& X,
		   const Vector<double, Vect_Full, Allocator2>& Y,
		   Matrix<double, Prop0, ColMajor, Allocator0>& A)
  {
    cblas_dger(CblasColMajor, A.GetM(), A.GetN(), alpha, X.GetData(), 1,
	       Y.GetData(), 1, A.GetData(), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator1>& X,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<float>, Prop0, ColMajor, Allocator0>& A)
  {
    cblas_cgeru(CblasColMajor, A.GetM(), A.GetN(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator1>& X,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<double>, Prop0, ColMajor, Allocator0>& A)
  {
    cblas_zgeru(CblasColMajor, A.GetM(), A.GetN(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  /*** ColMajor and ConjY ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator1>& X,
		   const SeldonConjugate& ConjY,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<float>, Prop0, ColMajor, Allocator0>& A)
  {
    if (ConjY.Conj())
      cblas_cgerc(CblasColMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
    else
      cblas_cgeru(CblasColMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator1>& X,
		   const SeldonConjugate& ConjY,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<double>, Prop0, ColMajor, Allocator0>& A)
  {
    if (ConjY.Conj())
      cblas_zgerc(CblasColMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
    else
      cblas_zgeru(CblasColMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  /*** RowMajor ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator1>& X,
		   const Vector<float, Vect_Full, Allocator2>& Y,
		   Matrix<float, Prop0, RowMajor, Allocator0>& A)
  {
    cblas_sger(CblasRowMajor, A.GetM(), A.GetN(), alpha, X.GetData(), 1,
	       Y.GetData(), 1, A.GetData(), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator1>& X,
		   const Vector<double, Vect_Full, Allocator2>& Y,
		   Matrix<double, Prop0, RowMajor, Allocator0>& A)
  {
    cblas_dger(CblasRowMajor, A.GetM(), A.GetN(), alpha, X.GetData(), 1,
	       Y.GetData(), 1, A.GetData(), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator1>& X,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<float>, Prop0, RowMajor, Allocator0>& A)
  {
    cblas_cgeru(CblasRowMajor, A.GetM(), A.GetN(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator1>& X,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<double>, Prop0, RowMajor, Allocator0>& A)
  {
    cblas_zgeru(CblasRowMajor, A.GetM(), A.GetN(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  /*** RowMajor and ConjY ***/


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator1>& X,
		   const SeldonConjugate& ConjY,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<float>, Prop0, RowMajor, Allocator0>& A)
  {
    if (ConjY.Conj())
      cblas_cgerc(CblasRowMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
    else
      cblas_cgeru(CblasRowMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  template <class Prop0, class Allocator0,
	    class Allocator1, class Allocator2>
  void Rank1Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator1>& X,
		   const SeldonConjugate& ConjY,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<double>, Prop0, RowMajor, Allocator0>& A)
  {
    if (ConjY.Conj())
      cblas_zgerc(CblasRowMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
    else
      cblas_zgeru(CblasRowMajor, A.GetM(), A.GetN(),
		  reinterpret_cast<const void*>(&alpha),
		  reinterpret_cast<const void*>(X.GetData()), 1,
		  reinterpret_cast<const void*>(Y.GetData()), 1,
		  reinterpret_cast<void*>(A.GetData()), A.GetM());
  }


  /*** ColSymPacked and Upper ***/


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   Matrix<float, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_sspr(CblasColMajor, CblasUpper, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   Matrix<double, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_dspr(CblasColMajor, CblasUpper, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   Matrix<complex<float>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_chpr(CblasColMajor, CblasUpper, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   Matrix<complex<double>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_zhpr(CblasColMajor, CblasUpper, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  /*** ColSymPacked and Uplo ***/


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<float, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_sspr(CblasColMajor, Uplo, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<double, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_dspr(CblasColMajor, Uplo, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<float>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_chpr(CblasColMajor, Uplo, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<double>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_zhpr(CblasColMajor, Uplo, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  /*** RowSymPacked and Upper ***/


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   Matrix<float, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_sspr(CblasRowMajor, CblasUpper, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   Matrix<double, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_dspr(CblasRowMajor, CblasUpper, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   Matrix<complex<float>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_chpr(CblasRowMajor, CblasUpper, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   Matrix<complex<double>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_zhpr(CblasRowMajor, CblasUpper, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  /*** RowSymPacked and Uplo ***/


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<float, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_sspr(CblasRowMajor, Uplo, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<double, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_dspr(CblasRowMajor, Uplo, A.GetM(), alpha,
	       X.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const float alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<float>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_chpr(CblasRowMajor, Uplo, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1>
  void Rank1Update(const double alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<double>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_zhpr(CblasRowMajor, Uplo, A.GetM(), alpha,
	       reinterpret_cast<const void*>(X.GetData()), 1,
	       reinterpret_cast<void*>(A.GetData()));
  }


  // RANK1UPDATE //
  /////////////////



  /////////////////
  // RANK2UPDATE //


  /*** ColSymPacked and Upper ***/


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   const Vector<float, Vect_Full, Allocator2>& Y,
		   Matrix<float, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_sspr2(CblasColMajor, CblasUpper, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   const Vector<double, Vect_Full, Allocator2>& Y,
		   Matrix<double, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_dspr2(CblasColMajor, CblasUpper, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<float>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_chpr2(CblasColMajor, CblasUpper, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<double>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_zhpr2(CblasColMajor, CblasUpper, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  /*** ColSymPacked and Uplo ***/


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   const Vector<float, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<float, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_sspr2(CblasColMajor, Uplo, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   const Vector<double, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<double, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_dspr2(CblasColMajor, Uplo, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<float>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_chpr2(CblasColMajor, Uplo, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<double>, Prop1, ColSymPacked, Allocator1>& A)
  {
    cblas_zhpr2(CblasColMajor, Uplo, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  /*** RowSymPacked and Upper ***/


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   const Vector<float, Vect_Full, Allocator2>& Y,
		   Matrix<float, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_sspr2(CblasRowMajor, CblasUpper, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   const Vector<double, Vect_Full, Allocator2>& Y,
		   Matrix<double, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_dspr2(CblasRowMajor, CblasUpper, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<float>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_chpr2(CblasRowMajor, CblasUpper, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   Matrix<complex<double>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_zhpr2(CblasRowMajor, CblasUpper, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  /*** RowSymPacked and Uplo ***/


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const float alpha,
		   const Vector<float, Vect_Full, Allocator0>& X,
		   const Vector<float, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<float, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_sspr2(CblasRowMajor, Uplo, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const double alpha,
		   const Vector<double, Vect_Full, Allocator0>& X,
		   const Vector<double, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<double, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_dspr2(CblasRowMajor, Uplo, A.GetM(), alpha,
		X.GetData(), 1, Y.GetData(), 1, A.GetData());
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<float> alpha,
		   const Vector<complex<float>, Vect_Full, Allocator0>& X,
		   const Vector<complex<float>, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<float>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_chpr2(CblasRowMajor, Uplo, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  // Warning: the matrix is assumed to be hermitian.
  template <class Allocator0,
	    class Prop1, class Allocator1,
	    class Allocator2>
  void Rank2Update(const complex<double> alpha,
		   const Vector<complex<double>, Vect_Full, Allocator0>& X,
		   const Vector<complex<double>, Vect_Full, Allocator2>& Y,
		   const enum CBLAS_UPLO Uplo,
		   Matrix<complex<double>, Prop1, RowSymPacked, Allocator1>& A)
  {
    cblas_zhpr2(CblasRowMajor, Uplo, A.GetM(),
		reinterpret_cast<const void*>(&alpha),
		reinterpret_cast<const void*>(X.GetData()), 1,
		reinterpret_cast<const void*>(Y.GetData()), 1,
		reinterpret_cast<void*>(A.GetData()));
  }


  // RANK2UPDATE //
  /////////////////


} // namespace Seldon

#define FILE_BLAS_2_CXX
#endif
